@@ -145,7 +145,8 @@ class Requestant(httping.Parsent):
         self.scheme = pathSplits.scheme
         self.hostname = pathSplits.hostname
         self.port = pathSplits.port
-        self.query = httping.unquoteQuery(pathSplits.query)  # unquote only the values
+        self.query = pathSplits.query  # WSGI spec leaves it quoted do not unquote
+        #self.query = httping.unquoteQuery(pathSplits.query)  # unquote only the values
         self.fragment = pathSplits.fragment
 
         leaderParser = httping.parseLeader(raw=self.msg,
@@ -294,18 +295,18 @@ class Responder(object):
             environ = wsgi environment dict
             chunkable = True if may send body in chunks
         """
+        status = "200 OK"  # integer or string with reason, WSGI is string with reason
         self.incomer = incomer
         self.app = app
         self.environ = environ
         self.chunkable = True if chunkable else False
-
         self.started = False  # True once start called (start_response)
         self.headed = False  # True once headers sent
         self.chunked = False  # True if should send in chunks
         self.ended = False  # True if response body completely sent
         self.closed = False  # True if connection closed by far side
         self.iterator = None  # iterator on application body
-        self.status = 200  # integer or string with reason
+        self.status = status
         self.headers = lodict()  # headers
         self.length = None  # if content-length provided must not exceed
         self.size = 0  # number of body bytes sent so far
@@ -331,7 +332,7 @@ class Responder(object):
         self.chunked = False
         self.ended = False
         self.iterator = None
-        self.status = 200
+        self.status = "200 OK"
         self.headers = lodict()
         self.length = None
         self.size = 0
@@ -343,10 +344,11 @@ class Responder(object):
         """
         lines = []
 
-        if isinstance(self.status, (int, long)):
-            status = "{0} {1}".format(self.status, httping.STATUS_DESCRIPTIONS[self.status])
-        else:
-            status = self.status
+        _status = getattr(self.iterator, '_status', None)  # if AttributiveGenerator
+        status = _status if _status is not None else self.status  # override
+
+        if isinstance(status, (int, long)):
+            status = "{0} {1}".format(status, httping.STATUS_DESCRIPTIONS[status])
 
         startLine = "{0} {1}".format(self.HttpVersionString, status)
         try:
@@ -355,6 +357,9 @@ class Responder(object):
             startLine = startLine.encode('idna')
         lines.append(startLine)
 
+        # Override if AttributiveGenerator
+        self.headers.update(getattr(self.iterator, '_headers', lodict()))
+
         if u'server' not in self.headers:  # create Server header
             self.headers[u'server'] = "Ioflo WSGI Server"
 
@@ -429,6 +434,14 @@ class Responder(object):
                     raise exc_info[0], exc_info[1], exc_info[2] (python2)
                     Use six.reraise to work for both
 
+        Nonstandard modifiction to allow for iterable/generator of body to change
+           headers and status before first write to support async processing of
+           responses whose iterator/generator yields empty before first non-empty
+           yield.  In .service yielding empty does not cause write so status line
+           and headers are not sent until first non-empty write.
+
+           The mode is that the app.headers and app.status are consulted to see
+           if changed from when .start = wsgi start_response was first called.
         """
         if exc_info:
             try:
@@ -463,34 +476,34 @@ class Responder(object):
         if not self.closed and not self.ended:
             if self.iterator is None:  # initiate application
                 self.iterator = iter(self.app(self.environ, self.start))
-
             try:
                 msg = next(self.iterator)
-            except StopIteration:
-                self.write(b'')  # if chunked send empty chunk to terminate
+            except StopIteration as ex:
+                if hasattr(ex, "value") and ex.value:
+                    self.write(ex.value)  # new style generators in python3.3+
+                self.write(b'')  # in case chunked send empty chunk to terminate
                 self.ended = True
-            except Exception as ex:  # handle http exceptions not caught by app
-                if not self.headed and hasattr(ex, 'status'):
-                    # assumes exception looks like bottle HTTPError
+            except httping.HTTPError as ex:
+                if not self.headed:
                     headers = lodict()
-                    if hasattr(ex, 'headerlist'):
-                        headers.update(ex.headerlist)
+                    headers.update(ex.headers.items())
                     if 'content-type' not in headers:
                         headers['content-type'] = 'text/plain'
-                    msg = b''
-                    if hasattr(ex, 'body'):
-                        msg = ex.body
-                        if isinstance(msg, unicode):
-                            msg = msg.encode('iso-8859-1')
+                    msg = ex.render()
                     headers['content-length'] = str(len(msg))
-                    self.start(ex.status, headers.items(), sys.exc_info())
+                    # WSGI status is string of status code and reason
+                    status = "{} {}".format(ex.status, ex.reason)
+                    self.start(status, headers.items(), sys.exc_info())
                     self.write(msg)
                     self.ended = True
                 else:
-                    raise
-
+                    console.terse("HTTPError streaming body after headers sent.\n"
+                                    "{}\n".format(ex))
+            except Exception as ex:  # handle http exceptions not caught by app
+                console.terse("Unexcepted Server Error.\n"
+                                    "{}\n".format(ex))
             else:
-                if msg:
+                if msg:  # only write if not empty allows async processing
                     self.write(msg)
                     if self.length is not None and self.size >= self.length:
                         self.ended = True
@@ -503,11 +516,11 @@ class Valet(object):
     Timeout = 5.0  # default server connection timeout
 
     def __init__(self,
+                 store=None,
                  app=None,
                  reqs=None,
                  reps=None,
                  servant=None,
-                 store=None,
                  name='',
                  bufsize=8096,
                  wlog=None,
@@ -520,30 +533,45 @@ class Valet(object):
                  **kwa):
         """
         Initialization method for instance.
-        app = wsgi application callable
-        reqs = odict of Requestant instances keyed by ca
-        apps = odict of running Wsgi Responder instances keyed by ca
-        servant = instance of Server or ServerTls or None
-        store = Datastore for timers
 
-        kwa needed to pass additional parameters to servant
+        Parameters:
+            store is Datastore for timers
+            app is wsgi application callable
+            reqs is odict of Requestant instances keyed by ca
+            reps is odict of running Wsgi Responder instances keyed by ca
+            servant is instance of Server or ServerTls or None
 
-        if servantinstances are not provided (None)
-        some or all of these parameters will be used for initialization
+            name is user friendly name for servant
+            bufsize is buffer size for servant
+            wlog is WireLog instance if any for servant
+            ha is  host address duple (host, port) for local servant listen socket
+            host is host address for local servant listen socket,
+                '' means any interface on host
+            port is socket port for local servant listen socket
+            eha is external destination address for servant
+                for incoming connections used in TLS
+            scheme is http scheme u'http' or u'https' or empty
+                for servant and WSGI environment
+            kwa needed to pass additional parameters to servant
 
-        name = user friendly name for servant
-        bufsize = buffer size
-        wlog = WireLog instance if any
-        ha = host address duple (host, port) for local servant listen socket
-        host = host address for local servant listen socket, '' means any interface on host
-        port = socket port for local servant listen socket
-        eha = external destination address for incoming connections used in TLS
-        scheme = http scheme u'http' or u'https' or empty
+            timeout is timeout in seconds for dropping idle connections
+
+        Attributes:
+            .store is Datastore for timers
+            .app is wsgi application callable
+            .reqs is odict of Requestant instances keyed by ca
+            .reps is odict of running Wsgi Responder instances keyed by ca
+            .servant is instance of Server or ServerTls or None
+            .timeout is timeout in seconds for dropping idle connections
+            .scheme is http scheme http or https for servant and environment
+            .secured is Boolean true if TLS
 
         """
         self.app = app
-        self.reqs = reqs if reqs is not None else odict()
-        self.reps = reps if reps is not None else odict()
+        self.reqs = reqs if reqs is not None else odict()  # allows external view
+        self.reqs.clear()  # items should only be assigned by valet
+        self.reps = reps if reps is not None else odict()  # allows external view
+        self.reps.clear()  # items should only be assigned by valet
         self.store = store or storing.Store(stamp=0.0)
         if not name:
             name = "Ioflo_WSGI_server"
@@ -583,6 +611,8 @@ class Valet(object):
             if servant.ha != ha:
                 ValueError("Provided ha '{0}:{1}' incompatible with servant".format(ha[0], ha[1]))
             # at some point may want to support changing the ha of provided servant
+            if name:
+                servant.name = name
 
         else:  # what about timeouts for servant connections
             if secured:
@@ -608,6 +638,19 @@ class Valet(object):
         self.secured = secured
         self.servant = servant
 
+    def open(self):
+        """
+        Return result of .servant.reopen()
+        """
+        return self.servant.reopen()
+
+    def close(self):
+        """
+        Call .servant.closeAll()
+        """
+        self.servant.closeAll()
+
+
     def idle(self):
         """
         Returns True if no connections have requests in process
@@ -688,7 +731,7 @@ class Valet(object):
                 self.closeConnection(ca)
                 continue
 
-            if ca not in self.reqs:
+            if ca not in self.reqs:  # point requestant.msg to incomer.rxbs
                 self.reqs[ca] = Requestant(msg=ix.rxbs, incomer=ix)
 
             if ix.timeout > 0.0 and ix.timer.expired:
@@ -702,7 +745,7 @@ class Valet(object):
             if requestant.parser:
                 try:
                     requestant.parse()
-                except httping.HTTPException as ex:
+                except httping.HTTPException as ex:  # this may be superfluous
                     #requestant.errored = True
                     #requestant.error = str(ex)
                     #requestant.ended = True
@@ -711,6 +754,11 @@ class Valet(object):
                     continue
 
                 if requestant.ended:
+                    if requestant.errored:  # parse may swallow error but set .errored and .error
+                        sys.stderr.write(requestant.error)
+                        self.closeConnection(ca)
+                        continue
+
                     console.concise("Parsed Request:\n{0} {1} {2}\n"
                                     "{3}\n{4}\n".format(requestant.method,
                                                         requestant.path,
@@ -734,15 +782,15 @@ class Valet(object):
         """
         Service pending responders
         """
-        for ca, respondent in self.reps.items():
-            if respondent.closed:
+        for ca, responder in self.reps.items():
+            if responder.closed:
                 self.closeConnection(ca)
                 continue
 
-            if not respondent.ended:
-                respondent.service()
+            if not responder.ended:
+                responder.service()
 
-            if respondent.ended:
+            if responder.ended:
                 requestant = self.reqs[ca]
                 if requestant.persisted:
                     if requestant.parser is None:  # reuse
